#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <math.h>
#include <stdbool.h>
#include <unistd.h>
#include <termios.h>
#include <limits.h>
#include <time.h>
#include <sys/time.h>
#include <SDL2/SDL.h>
#include <SDL2/SDL_image.h>

#include "structure.h"
#include "base.h"
#include "display.h"
#include "collisions.h"
#include "rooms.h"
#include "cars.h"

const int N_PLAYERS = 6;
double elapsed = 0.0;

bool halt = false;
void playerActions() {
    SDL_Event event;
    while(SDL_PollEvent(&event)) {
        switch (event.type) {
            case SDL_QUIT:
                break;
            case SDL_KEYDOWN:
                switch (event.key.keysym.sym) {
                    case SDLK_SPACE:
                        halt = true;
                        break;

                    default:
                        break;
                }
        }
    }
}

int main() {
    srand(time(NULL));
    if (SDL_Init(SDL_INIT_EVERYTHING) != 0) {
        printf("error initializing SDL: %s\n", SDL_GetError());
    }
    SDL_Window* win = SDL_CreateWindow("Game",
                                       SDL_WINDOWPOS_CENTERED,
                                       SDL_WINDOWPOS_CENTERED,
                                       WIDTH, HEIGHT, 0);
 
    Uint32 render_flags = SDL_RENDERER_ACCELERATED;
    SDL_Renderer* rend = SDL_CreateRenderer(win, -1, render_flags);
    SDL_SetRenderDrawBlendMode(rend, SDL_BLENDMODE_BLEND);

    import_letters(rend);
    import_digits(rend);
    // ---------------------------- //
    //test_file("levels/test.txt");
    level* test = parse_level("levels/test.txt");
    init_all(N_PLAYERS);

    printf("Starting direction : ");
    if(START_DIR == 0) {
        printf("North\n");
    } else if(START_DIR == 1) {
        printf("East\n");
    } else if(START_DIR == 2) {
        printf("South\n");
    } else if(START_DIR == 3) {
        printf("West\n");
    } else {
        printf("???\n");
    }

    while(!halt) {
        resetRenderer(rend);
        renderMap(rend, test, 1, 1, 2, 250);
        if(elapsed <= 0.7) {
            placeRectToRenderer(rend, 0, 0, 50, 50, 255, 255, 32, 192);
        } else if(updateCars(test)) {
            placeRectToRenderer(rend, 0, 0, 50, 50, 32, 255, 32, 192);
        } else {
            placeRectToRenderer(rend, 0, 0, 50, 50, 255, 32, 32, 192);
        }
        updateRenderer(rend);
        elapsed += DT;
        //for(int p = 0; p < N_PLAYERS; p++) {
        //    printf("[%d], (at [%d %d] (%lf %lf) (%lf %lf))\n", p, players[p].c->chx, players[p].c->chy, players[p].c->pos.fx, players[p].c->pos.fy, players[p].c->vel.fx, players[p].c->vel.fy);
        //}
        //printf("\n");
        usleep((int)(1000000*DT));
        playerActions();
    }
    
    // ---------------------------- //
    free_digits(digits);
    free_digits(letters);
    free_level(test);

    SDL_DestroyRenderer(rend);
    SDL_DestroyWindow(win);
    SDL_Quit();
    return 0;
}

/* TODO :
    - add collisions (4/10)
    - add level generation
    - add player input parsing (and some dumb code to test it)
    - add function that gives players the necessary data to play
    - check seg faults
    + add items
    + add hazards
*/